﻿Imports System
Imports novapiLib90

Public Class NovaTools

    '------------------------------------------------------------------------------------------'
    '-------------------------------------  BOOKMARKS ----------------------------------------'
    '------------------------------------------------------------------------------------------'
    Public Shared Sub AddBookmarksDefinitions(ByVal pNova As INovaPdfOptions90)

        Dim nDefinition, nCount As Short

        'Enable bookmarks
        pNova.SetOptionBool(NovaOptions.NOVAPDF_BMARK_EN_AUTO_DET_BMARKS, 1)
        'novaPDF driver will serach the text in the printed document, and if some text matches a bookmark definition
        '(for font, style, ssize, color) a bookmark with this text will be added in the pdf file.
        'if default profile contains some definitions, delete them
        pNova.GetBookmarkDefinitionCount(nCount)
        For I As Short = nCount - 1 To 0
            pNova.DeleteBookmarkDefinition(I)
        Next
        'add new definition
        pNova.AddBookmarkDefinition(1, 1, 1, "Batang", FontType.NOVA_FONTTYPE_TRUETYPE, 1, 1, 0, 1, 22, (0.4), 0, System.Drawing.Color.Blue.ToArgb(), 1, 0, System.Drawing.Color.Blue.ToArgb(), nDefinition)
        'pNova.DeleteBookmarkDefinition(nDefinition)
        pNova.ModifyBookmarkDefinition(nDefinition, 1, 1, 1, "Courier", 1, 1, 1, 0, 1, 22, (0.7), 0, 0, 1, 0, 0)
        'Change general bookmarks settings
        pNova.SetOptionBool(NovaOptions.NOVAPDF_BMK_ALLOW_MULTILINE, 1)
        pNova.SetOptionBool(NovaOptions.NOVAPDF_BMK_MATCH_ALL_LEVELS, 1)
        pNova.SetOptionLong(NovaOptions.NOVAPDF_BMK_USE_LEVELS, 1)
        pNova.SetOptionLong(NovaOptions.NOVAPDF_BMK_OPEN_TO_LEVEL, 1)
        'Add root bookmark
        pNova.SetOptionBool(NovaOptions.NOVAPDF_BMK_ROOT_ENABLED, 1)
        pNova.SetOptionString2(NovaOptions.NOVAPDF_BMK_ROOT_NAME, "[T]")
        pNova.SetOptionBool(NovaOptions.NOVAPDF_BMK_ROOT_BOLD, 0)
        pNova.SetOptionBool(NovaOptions.NOVAPDF_BMK_ROOT_ITALIC, 1)
        pNova.SetOptionLong(NovaOptions.NOVAPDF_BMK_ROOT_COLOR, 0)

    End Sub

    '------------------------------------------------------------------------------------------'
    '--------------------------------  DOCUMENT INFORMATION  ----------------------------------'
    '------------------------------------------------------------------------------------------'
    Public Shared Sub AddDocumentInformation(ByVal pNova As INovaPdfOptions90)
        pNova.SetOptionBool(NovaOptions.NOVAPDF_INFO_ENABLE, 1)
        'show a dialog before printing each document to enter doc info
        pNova.SetOptionBool(NovaOptions.NOVAPDF_SHOW_DOCINFO_DLG, 0)
        'pdf document information
        pNova.SetOptionString(NovaOptions.NOVAPDF_DOCINFO_TITLE, "My Doc Title")
        pNova.SetOptionString2(NovaOptions.NOVAPDF_DOCINFO_SUBJECT, "My Doc Subject")
        pNova.SetOptionString2(NovaOptions.NOVAPDF_DOCINFO_AUTHOR, "My Doc Author")
        pNova.SetOptionString2(NovaOptions.NOVAPDF_DOCINFO_CREATOR, "My Doc Creator")
        pNova.SetOptionString2(NovaOptions.NOVAPDF_DOCINFO_KEYWORDS, "keyword1, keyword2,...")
    End Sub

    '------------------------------------------------------------------------------------------'
    '--------------------------------  DOCUMENT METADATA --------------------------------------'
    '------------------------------------------------------------------------------------------'
    Public Shared Sub AddDocumentXMPMetadata(ByVal pNova As INovaPdfOptions90)
        pNova.SetOptionBool(NovaOptions.NOVAPDF_ADD_XMP_METADATA, 1)
        'copyright information
        pNova.SetOptionString2(NovaOptions.NOVAPDF_COPYRIGHT_TEXT, "My Doc Copyright Text")
        pNova.SetOptionString2(NovaOptions.NOVAPDF_COPYRIGHT_URL, "My Doc Copyright URL")
        pNova.SetOptionLong(NovaOptions.NOVAPDF_COPYRIGHT_TYPE, CopyrightType.COPYRIGHT_TYPE_COPYRIGHTED)
    End Sub

    '------------------------------------------------------------------------------------------'
    '----------------------------------------  SECURITY  --------------------------------------'
    '------------------------------------------------------------------------------------------'
    Public Shared Sub AddSecurityOptions(ByVal pNova As INovaPdfOptions90)
        'first enable security - mandatory
        pNova.SetOptionBool(NovaOptions.NOVAPDF_SEC_ENABLE, 1)
        'if a dialog should be shown before printing wach document, to change password
        pNova.SetOptionBool(NovaOptions.NOVAPDF_SHOW_SECURITY_DLG, 0)
        'level
        pNova.SetOptionLong(NovaOptions.NOVAPDF_SECURITY_LEVEL, SecurityLevel.SECURITY_128BITS)
        'then set security flags: 1 if it is allowed, 0 if not
        'print document
        pNova.SetOptionBool(NovaOptions.NOVAPDF_SECURITY_FLAG_PRINTDOC, 1)
        'modify document content
        pNova.SetOptionBool(NovaOptions.NOVAPDF_SECURITY_FLAG_MODIFYDOC, 1)
        'extract text and grapics
        pNova.SetOptionBool(NovaOptions.NOVAPDF_SECURITY_FLAG_EXTRACTTEXT, 1)
        'modify annotations, fill forms
        pNova.SetOptionBool(NovaOptions.NOVAPDF_SECURITY_FLAG_MODIFYANNOTATION, 1)
        'advanced fill forms
        pNova.SetOptionBool(NovaOptions.NOVAPDF_SECURITY_FLAG_ADVANCEDFILL, 0)
        'advanced extract text and graphics
        pNova.SetOptionBool(NovaOptions.NOVAPDF_SECURITY_FLAG_ADVANCEDEXTRACT, 0)
        'assemble the document
        pNova.SetOptionBool(NovaOptions.NOVAPDF_SECURITY_FLAG_ASSEMBLEDOC, 0)
        'print at high resolution
        pNova.SetOptionBool(NovaOptions.NOVAPDF_SECURITY_FLAG_PRINTHIGHRES, 0)

        'passwords are set as encrypted strings (owner password is mandatory, user password is optional)
        'user password for opening the PDF
        pNova.SetOptionEncryptedString(NovaOptions.NOVAPDF_SECURITY_USER_PASSWORD, "my user password")
        'owner password for changing PDF security
        pNova.SetOptionEncryptedString(NovaOptions.NOVAPDF_SECURITY_OWNER_PASSWORD, "my owner password")
    End Sub


    '------------------------------------------------------------------------------------------'
    '--------------------------------------  DETECT LINKS  ------------------------------------'
    '------------------------------------------------------------------------------------------'
    Public Shared Sub AddLinksOptions(ByVal pNova As INovaPdfOptions90)
        'detect link / URL options
        'first enable links option - mandatory
        pNova.SetOptionBool(NovaOptions.NOVAPDF_URL_ANALIZE, 1)
        'flag to detect or not file links
        pNova.SetOptionBool(NovaOptions.NOVAPDF_URL_DETECT_FILES, 1)
        'when detecting file links, check if the file actually exists (1-check, 0-do not check)
        pNova.SetOptionBool(NovaOptions.NOVAPDF_URL_CHECK_FILE_EXISTS, 1)
        'how to show the links in the pdf file
        pNova.SetOptionBool(NovaOptions.NOVAPDF_URL_UNDERLINE, 1)
        'set links color blue
        pNova.SetOptionBool(NovaOptions.NOVAPDF_URL_OVERWRITE_COLOR, 1)
        pNova.SetOptionLong(NovaOptions.NOVAPDF_URL_COLOR, System.Drawing.Color.Blue.ToArgb())
    End Sub


    '------------------------------------------------------------------------------------------'
    '--------------------------------------  FONTS OPTIONS  -----------------------------------'
    '------------------------------------------------------------------------------------------'
    Public Shared Sub AddEmbedFontsOptions(ByVal pNova As INovaPdfOptions90)
        'embed all used fionts in the pdf file 
        pNova.SetOptionBool(NovaOptions.NOVAPDF_FONTS_EMBED_ALL_USED_FONTS, 1)
        'embed only a subset of the fonts (only the characters that are used in the document)
        'embed all fonts has to be set too, otherwise fonts will not be embedded at all
        pNova.SetOptionBool(NovaOptions.NOVAPDF_FONTS_EMBED_SUBSETS, 1)
        'some fonts cannot be embedded because their license does not allow the font to be embedded
        'but with the next option you can force to embed licensed fonts
        pNova.SetOptionBool(NovaOptions.NOVAPDF_FONTS_FORCE_EMBED_PROTECTED, 1)
        'if we have some specific fonts that we wish to add to embed/ not emebed lists
        pNova.ClearFontOptions()
        pNova.SetFontOption("Arial", 1, 0)
        pNova.SetFontOption("Verdana", 0, 1)
        pNova.SetFontOption("Courier", 0, 1)
    End Sub

    '------------------------------------------------------------------------------------------'
    '------------------------------------  GRAPHICS OPTIONS  ----------------------------------'
    '------------------------------------------------------------------------------------------'
    Public Shared Sub AddGraphicsOptions(ByVal pNova As INovaPdfOptions90)
        'enable image compression
        pNova.SetOptionBool(NovaOptions.NOVAPDF_COMPRESS_ENABLE, 1)
        ' compression settings for high color images - flag, method, level 
        pNova.SetOptionBool(NovaOptions.NOVAPDF_COMPRESS_HIGH_COLOR, 1)
        pNova.SetOptionLong(NovaOptions.NOVAPDF_COMPRESS_HIGH_COLOR_TYPE, CompressionMethod.COMPRESS_METHOD_JPEG)
        pNova.SetOptionLong(NovaOptions.NOVAPDF_COMPRESS_HIGH_COLOR_LEVEL, System.Drawing.Color.Blue.ToArgb())
        ' compression settings for indexed images - flag, method, level 
        pNova.SetOptionBool(NovaOptions.NOVAPDF_COMPRESS_INDEXED, 1)
        pNova.SetOptionLong(NovaOptions.NOVAPDF_COMPRESS_INDEXED_TYPE, CompressionMethod.COMPRESS_METHOD_ZIP)
        pNova.SetOptionLong(NovaOptions.NOVAPDF_COMPRESS_INDEXED_LEVEL, 5)
        ' compression settings for monochrome images - flag, method, level 
        pNova.SetOptionBool(NovaOptions.NOVAPDF_COMPRESS_MONOCHROM, 1)
        pNova.SetOptionLong(NovaOptions.NOVAPDF_COMPRESS_MONOCHROM_TYPE, CompressionMethod.COMPRESS_METHOD_ZIP)
        pNova.SetOptionLong(NovaOptions.NOVAPDF_COMPRESS_MONOCHROM_LEVEL, 5)
        ' compression settings for text and graphics - flag, method, level 
        pNova.SetOptionBool(NovaOptions.NOVAPDF_COMPRESS_TEXT_GRAPHICS, 1)
        pNova.SetOptionLong(NovaOptions.NOVAPDF_COMPRESS_TEXT_GRAPHICS_TYPE, CompressionMethod.COMPRESS_METHOD_ZIP)
        pNova.SetOptionLong(NovaOptions.NOVAPDF_COMPRESS_TEXT_GRAPHICS_LEVEL, 5)

        'enable downsample
        pNova.SetOptionBool(NovaOptions.NOVAPDF_GR_DOWNSMPL_ENABLE, 1)
        'downsample high color images - flag, dpi, method
        pNova.SetOptionBool(NovaOptions.NOVAPDF_DOWNSAMPLE_HIGH_COLOR, 1)
        pNova.SetOptionLong(NovaOptions.NOVAPDF_DOWNSAMPLE_RES_HIGH_COLOR, 96)
        pNova.SetOptionLong(NovaOptions.NOVAPDF_DOWNSAMPLE_TYPE_HIGH_COLOR, DownsampleFilter.DOWNSAMPLE_FILTER_BICUBIC)
        'downsample indexed images - flag, dpi, method
        pNova.SetOptionBool(NovaOptions.NOVAPDF_DOWNSAMPLE_INDEXED, 1)
        pNova.SetOptionLong(NovaOptions.NOVAPDF_DOWNSAMPLE_RES_INDEXED, 96)
        pNova.SetOptionLong(NovaOptions.NOVAPDF_DOWNSAMPLE_TYPE_INDEXED, DownsampleFilter.DOWNSAMPLE_FILTER_BICUBIC)
        'downsample monochrome images - flag, dpi, method
        pNova.SetOptionBool(NovaOptions.NOVAPDF_DOWNSAMPLE_MONOCHROM, 1)
        pNova.SetOptionLong(NovaOptions.NOVAPDF_DOWNSAMPLE_RES_MONOCHROM, 96)
        pNova.SetOptionLong(NovaOptions.NOVAPDF_DOWNSAMPLE_TYPE_MONOCHROM, DownsampleFilter.DOWNSAMPLE_FILTER_BICUBIC)

        'enable image convertion
        pNova.SetOptionBool(NovaOptions.NOVAPDF_GR_CONVERT_ENABLE, 1)
        'convert high color images - flag, type
        pNova.SetOptionBool(NovaOptions.NOVAPDF_IMAGE_CONVERT_HIGH_COLOR, 1)
        pNova.SetOptionLong(NovaOptions.NOVAPDF_IMAGE_CONVERT_TYPE_HIGH_COLOR, ImageConversionType.CONVERT_GRAYSCALE)
        'if convert to monochrome, set trashold color and dithering method
        pNova.SetOptionLong(NovaOptions.NOVAPDF_IMAGE_CONVERT_TRESH_HIGH_COLOR, 128)
        'use or no dithering
        pNova.SetOptionBool(NovaOptions.NOVAPDF_IMAGE_DITHER_HIGH_COLOR, 1)
        pNova.SetOptionLong(NovaOptions.NOVAPDF_IMAGE_DITHER_TYPE_HIGH_COLOR, Dither.DITHER_BAYER4)
        'convert indexed images - flag, type
        pNova.SetOptionBool(NovaOptions.NOVAPDF_IMAGE_CONVERT_INDEXED, 1)
        pNova.SetOptionLong(NovaOptions.NOVAPDF_IMAGE_CONVERT_TYPE_INDEXED, ImageConversionType.CONVERT_GRAYSCALE)
        'if convert to monochrome, set trashold color and dithering method
        pNova.SetOptionLong(NovaOptions.NOVAPDF_IMAGE_CONVERT_TRESH_INDEXED, 128)
        'use or no dithering
        pNova.SetOptionBool(NovaOptions.NOVAPDF_IMAGE_DITHER_INDEXED, 1)
        pNova.SetOptionLong(NovaOptions.NOVAPDF_IMAGE_DITHER_TYPE_INDEXED, Dither.DITHER_BAYER4)
        'convert text and graphics colors - flag, type
        pNova.SetOptionBool(NovaOptions.NOVAPDF_IMAGE_CONVERT_TEXTANDGR, 1)
        pNova.SetOptionLong(NovaOptions.NOVAPDF_IMAGE_CONVERT_TYPE_TEXTANDGR, ImageConversionType.CONVERT_GRAYSCALE)
        'if convert to monochrome, set trashold color 
        pNova.SetOptionLong(NovaOptions.NOVAPDF_IMAGE_CONVERT_TRESH_TEXTANDGR, 128)
    End Sub

    '------------------------------------------------------------------------------------------'
    '-------------------------------------  OVERLAY OPTIONS  ----------------------------------'
    '------------------------------------------------------------------------------------------'
    Public Shared Sub AddOverlayOptions(ByVal pNova As INovaPdfOptions90)
        'set the pdf file you wish to overlay with (with full path)
        pNova.SetOptionLong(NovaOptions.NOVAPDF_OVERLAY_FILE_LOCATION, LocationType.LOCATION_TYPE_LOCAL)
        'pNova->SetOptionString(NOVAPDF_OVERLAY_FILE, L"C:\... my pdf file")
        pNova.SetOptionString(NovaOptions.NOVAPDF_OVERLAY_FILE, "letterhead-template.pdf")
        'if the file is password protected, enter the password here
        pNova.SetOptionEncryptedString(NovaOptions.NOVAPDF_OVERLAY_FILE_PASSWORD, "")
        'if the overlay file is on a network path, set network user name and password to access this path
        pNova.SetOptionString(NovaOptions.NOVAPDF_OVERLAY_USER_NAME, "")
        pNova.SetOptionEncryptedString(NovaOptions.NOVAPDF_OVERLAY_USER_PASSWORD, "")

        'repeat overlay option
        pNova.SetOptionLong(NovaOptions.NOVAPDF_OVERLAY_REPEAT_TYPE, OverlayRepeatType.OVERLAY_REPEAT_ALL)
        'opacity
        pNova.SetOptionLong(NovaOptions.NOVAPDF_OVERLAY_OPACITY, 50)

        'LAYOUT
        Dim pwsOverlayId As String = vbNullString
        Dim pwsLayoutId As String = vbNullString
        'Get OVERLAY ID
        pNova.GetOverlay(pwsOverlayId, pwsLayoutId)

        'metadata
        pNova.SetLayoutOptionString(pwsOverlayId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_NAME, "Overlay Layout")
        pNova.SetLayoutOptionString(pwsOverlayId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_AUTHOR, "Hello World application")
        pNova.SetLayoutOptionString(pwsOverlayId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_DESCRIPTION, "overlay position on page")

        'anchors
        'percentage or thousands of mm
        pNova.SetLayoutOptionBool(pwsOverlayId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_LEFTANCHOR_USED, 1)
        pNova.SetLayoutOptionFloat(pwsOverlayId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_LEFTANCHOR_OFFSET, 0)
        pNova.SetLayoutOptionBool(pwsOverlayId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_LEFTANCHOR_USE_PERCENT, 0)

        pNova.SetLayoutOptionBool(pwsOverlayId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_RIGHTANCHOR_USED, 0)
        pNova.SetLayoutOptionFloat(pwsOverlayId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_RIGHTANCHOR_OFFSET, 0)
        pNova.SetLayoutOptionBool(pwsOverlayId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_RIGHTANCHOR_USE_PERCENT, 0)

        pNova.SetLayoutOptionBool(pwsOverlayId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_TOPANCHOR_USED, 1)
        pNova.SetLayoutOptionFloat(pwsOverlayId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_TOPANCHOR_OFFSET, 0)
        pNova.SetLayoutOptionBool(pwsOverlayId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_TOPANCHOR_USE_PERCENT, 0)

        pNova.SetLayoutOptionBool(pwsOverlayId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_BOTTOMANCHOR_USED, 0)
        pNova.SetLayoutOptionFloat(pwsOverlayId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_BOTTOMANCHOR_OFFSET, 0)
        pNova.SetLayoutOptionBool(pwsOverlayId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_BOTTOMANCHOR_USE_PERCENT, 0)

        pNova.SetLayoutOptionBool(pwsOverlayId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_VERTANCHOR_USED, 0)
        pNova.SetLayoutOptionFloat(pwsOverlayId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_VERTLANCHOR_OFFSET, 0)
        pNova.SetLayoutOptionBool(pwsOverlayId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_VERTANCHOR_USE_PERCENT, 0)

        pNova.SetLayoutOptionBool(pwsOverlayId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_HORIZANCHOR_USED, 0)
        pNova.SetLayoutOptionFloat(pwsOverlayId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_HORIZANCHOR_OFFSET, 0)
        pNova.SetLayoutOptionBool(pwsOverlayId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_HORIZANCHOR_USE_PERCENT, 0)

        'aspect ratio
        pNova.SetLayoutOptionBool(pwsOverlayId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_USE_ASPECT_RATIO, 0)
        'allow stretch with anchors when print on a different page size or layout
        pNova.SetLayoutOptionBool(pwsOverlayId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_ALLOW_STRETCH, 0)

        'rotation (in radians)
        pNova.SetLayoutOptionFloat(pwsOverlayId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_ROTATION, 0)

        'ZIndex (multiple of 5) - objects with higher ZIndex will be above the ones with smaller ZIndex
        pNova.SetLayoutOptionLong(pwsOverlayId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_ZINDEX, 10)

        'Border
        pNova.SetLayoutOptionBool(pwsOverlayId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_USE_BORDER, 1)
        pNova.SetLayoutOptionLong(pwsOverlayId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_BORDER_POSITION, BorderPosition.BORDER_POSITION_INTERIOR)
        pNova.SetLayoutOptionLong(pwsOverlayId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_BORDER_WIDTH, 2000)
        pNova.SetLayoutOptionLong(pwsOverlayId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_BORDER_STYLE, BorderStyle.BORDER_STYLE_DASHED)
        pNova.SetLayoutOptionLong(pwsOverlayId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_BORDER_COLOR, System.Drawing.Color.Green.ToArgb())

        'pages
        pNova.SetLayoutOptionBool(pwsOverlayId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_FIRSTPAGE, 0)
        pNova.SetLayoutOptionBool(pwsOverlayId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_LASTPAGE, 0)
        pNova.SetLayoutOptionBool(pwsOverlayId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_ODDPAGE, 0)
        pNova.SetLayoutOptionBool(pwsOverlayId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_EVENPAGE, 0)
        pNova.SetLayoutOptionBool(pwsOverlayId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_ALLPAGE, 1)
        pNova.SetLayoutOptionBool(pwsOverlayId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_PAGES, 0)
        pNova.SetLayoutOptionString(pwsOverlayId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_PAGERANGE, "1-2")
    End Sub

    '------------------------------------------------------------------------------------------
    '-----------------------------------  DIGITAL SIGNATURES  ---------------------------------
    '------------------------------------------------------------------------------------------
    Public Shared Sub AddSignatureOptions(ByVal pNova As INovaPdfOptions90)
        ''choose a certificate : a certificate installed in the system or from a file
        ''if you wish to set a system certificate:
        'pNova.SetOptionLong(NovaOptions.NOVAPDF_SIGN_CERTIFICATE_TYPE, CertificateType.CERTIFICATE_TYPE_SYSTEM)
        ''set system signature information so it can be recognized by novaPDF printer 
        ''this information can be retrieved from system using CertOpenSystemStore, CertEnumCertificatesInStore
        ''CertGetNameString - CERT_NAME_SIMPLE_DISPLAY_TYPE, CERT_NAME_SIMPLE_DISPLAY_TYPE
        'pNova.SetOptionString(NovaOptions.NOVAPDF_SIGN_SYSCERT_ISSUEDTO, "C=..., S=..., L=..., O=..., OU=..., E=..., CN=...")
        'pNova.SetOptionString(NovaOptions.NOVAPDF_SIGN_SYSCERT_ISSUEDBY, "C=..., S=..., L=..., O=..., OU=..., E=..., CN=...")
        'pNova.SetOptionString(NovaOptions.NOVAPDF_SIGN_SYSCERT_FRIENDLYNAME, "<friendly name>")
        ''pCertContext->pCertInfo->Subject
        'pNova.SetOptionString(NovaOptions.NOVAPDF_SIGN_SYSCERT_SUBJECT, "C=..., S=..., L=..., O=..., OU=..., E=..., CN=...")
        '    pNova.SetOptionString(NovaOptions.NOVAPDF_SIGN_SYSCERT_SUBJECT_NAME, "subject name");
        ''pCertContext->pCertInfo->NotAfter
        '	pNova.SetOptionString(NovaOptions.NOVAPDF_SIGN_SYSCERT_EXPIRE_DATE, L"2013-01-11T20:17:34Z")


        ''if you wish to set a certificate from file:
        pNova.SetOptionLong(NovaOptions.NOVAPDF_SIGN_CERTIFICATE_TYPE, CertificateType.CERTIFICATE_TYPE_FILE)
        ''certificate file and password
        pNova.SetOptionLong(NovaOptions.NOVAPDF_SIGN_FILECERT_LOCATION, LocationType.LOCATION_TYPE_LOCAL)
        pNova.SetOptionString(NovaOptions.NOVAPDF_SIGN_FILECERT_FILE, "sample.p12")
        ''if the file is password protected, enter the password here
        pNova.SetOptionEncryptedString(NovaOptions.NOVAPDF_SIGN_FILECERT_FILE_PASSWORD, "novapdf")
        ''if the folder is on a network path, set network user name and password to access this path
        pNova.SetOptionString(NovaOptions.NOVAPDF_SIGN_FILECERT_USER_NAME, "")
        pNova.SetOptionEncryptedString(NovaOptions.NOVAPDF_SIGN_FILECERT_USER_PASSWORD, "")

        'customize how the signature will look in the PDF file:
        'show or not the Name on the left 
        pNova.SetOptionBool(NovaOptions.NOVAPDF_SIGN_SHOW_GRAPHIC_NAME, 1)
        'show or not an image on the left 
        pNova.SetOptionBool(NovaOptions.NOVAPDF_SIGN_SHOW_GRAPHIC_IAMGE, 1)
        'name - opacity percentage
        pNova.SetOptionLong(NovaOptions.NOVAPDF_SIGN_NAME_OPACITY, 50)
        'image file
        pNova.SetOptionLong(NovaOptions.NOVAPDF_SIGN_IMAGE_LOCATION, LocationType.LOCATION_TYPE_LOCAL)
        pNova.SetOptionString(NovaOptions.NOVAPDF_SIGN_IMAGE_FILE, "nopreview.png")
        'if the image file is on a network path, set network user name and password to access this path
        pNova.SetOptionString(NovaOptions.NOVAPDF_SIGN_IMAGE_USER_NAME, "")
        pNova.SetOptionEncryptedString(NovaOptions.NOVAPDF_SIGN_IMAGE_USER_PASSWORD, "")
        'image - keep aspect ratio
        pNova.SetOptionBool(NovaOptions.NOVAPDF_SIGN_SHOW_KEEPASPECTRATIO, 1)
        'image - opcaity percentage
        pNova.SetOptionLong(NovaOptions.NOVAPDF_SIGN_IMAGE_OPACITY, 50)
        'which details to show on the right
        pNova.SetOptionBool(NovaOptions.NOVAPDF_SIGN_SHOW_LABELS, 0)
        pNova.SetOptionBool(NovaOptions.NOVAPDF_SIGN_SHOW_CERTIFICATE_NAME, 0)
        pNova.SetOptionBool(NovaOptions.NOVAPDF_SIGN_SHOW_SIGN_DATE, 0)
        pNova.SetOptionBool(NovaOptions.NOVAPDF_SIGN_SHOW_DETAILS, 1)
        pNova.SetOptionBool(NovaOptions.NOVAPDF_SIGN_SHOW_SIGN_LOCATION, 0)
        pNova.SetOptionBool(NovaOptions.NOVAPDF_SIGN_SHOW_SIGN_REASON, 0)
        pNova.SetOptionBool(NovaOptions.NOVAPDF_SIGN_SHOW_CONTACT_INFO, 0)
        pNova.SetOptionString(NovaOptions.NOVAPDF_SIGN_INFO_REASON, "<myreason>")
        pNova.SetOptionString(NovaOptions.NOVAPDF_SIGN_INFO_LOCATION, "<mylocation>")
        pNova.SetOptionString(NovaOptions.NOVAPDF_SIGN_INFO_CONTACT, "<mycontactinfo>")
        ' if you wish a custom font size on the right
        pNova.SetOptionString(NovaOptions.NOVAPDF_SIGN_FONT_NAME, "Arial")
        pNova.SetOptionLong(NovaOptions.NOVAPDF_SIGN_FONT_TYPE, FontType.NOVA_FONTTYPE_TRUETYPE)
        pNova.SetOptionBool(NovaOptions.NOVAPDF_SIGN_CUSTOM_SIZE, 0)
        pNova.SetOptionLong(NovaOptions.NOVAPDF_SIGN_FONT_SIZE, 8)
        ' font color
        pNova.SetOptionLong(NovaOptions.NOVAPDF_SIGN_NAME_COLOR, System.Drawing.Color.Blue.ToArgb())

        'visibility
        pNova.SetOptionBool(NovaOptions.NOVAPDF_SIGN_VIEW_VIEW, 1)
        pNova.SetOptionBool(NovaOptions.NOVAPDF_SIGN_VIEW_PRINT, 1)
        pNova.SetOptionBool(NovaOptions.NOVAPDF_SIGN_VIEW_EXPORT, 1)

        'LAYOUT
        Dim pwsSignatureId As String = ""
        Dim pwsLayoutId As String = ""
        'Get signature ID
        pNova.GetSignature(pwsSignatureId, pwsLayoutId)

        'metadata
        pNova.SetLayoutOptionString(pwsSignatureId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_NAME, "Signature Layout")
        pNova.SetLayoutOptionString(pwsSignatureId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_AUTHOR, "Hello World application")
        pNova.SetLayoutOptionString(pwsSignatureId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_DESCRIPTION, "signature position on page")

        'anchors
        'percentage or thousands of mm
        pNova.SetLayoutOptionBool(pwsSignatureId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_LEFTANCHOR_USED, 0)
        pNova.SetLayoutOptionFloat(pwsSignatureId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_LEFTANCHOR_OFFSET, 0)
        pNova.SetLayoutOptionBool(pwsSignatureId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_LEFTANCHOR_USE_PERCENT, 0)

        pNova.SetLayoutOptionBool(pwsSignatureId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_RIGHTANCHOR_USED, 0)
        pNova.SetLayoutOptionFloat(pwsSignatureId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_RIGHTANCHOR_OFFSET, 0)
        pNova.SetLayoutOptionBool(pwsSignatureId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_RIGHTANCHOR_USE_PERCENT, 0)

        pNova.SetLayoutOptionBool(pwsSignatureId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_TOPANCHOR_USED, 0)
        pNova.SetLayoutOptionFloat(pwsSignatureId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_TOPANCHOR_OFFSET, 0)
        pNova.SetLayoutOptionBool(pwsSignatureId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_TOPANCHOR_USE_PERCENT, 0)

        pNova.SetLayoutOptionBool(pwsSignatureId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_BOTTOMANCHOR_USED, 1)
        pNova.SetLayoutOptionFloat(pwsSignatureId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_BOTTOMANCHOR_OFFSET, 10000)
        pNova.SetLayoutOptionBool(pwsSignatureId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_BOTTOMANCHOR_USE_PERCENT, 0)

        pNova.SetLayoutOptionBool(pwsSignatureId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_VERTANCHOR_USED, 0)
        pNova.SetLayoutOptionFloat(pwsSignatureId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_VERTLANCHOR_OFFSET, 0)
        pNova.SetLayoutOptionBool(pwsSignatureId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_VERTANCHOR_USE_PERCENT, 0)

        pNova.SetLayoutOptionBool(pwsSignatureId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_HORIZANCHOR_USED, 1)
        pNova.SetLayoutOptionFloat(pwsSignatureId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_HORIZANCHOR_OFFSET, 0)
        pNova.SetLayoutOptionBool(pwsSignatureId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_HORIZANCHOR_USE_PERCENT, 0)

        'aspect ratio
        pNova.SetLayoutOptionBool(pwsSignatureId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_USE_ASPECT_RATIO, 0)
        'allow stretch with anchors when print on a different page size or layout
        pNova.SetLayoutOptionBool(pwsSignatureId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_ALLOW_STRETCH, 0)

        'Width / Height
        pNova.SetLayoutOptionFloat(pwsSignatureId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_OBJECT_WIDTH, 100000)
        pNova.SetLayoutOptionFloat(pwsSignatureId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_OBJECT_HEIGHT, 50000)

        'rotation (in Radians) - the signature is not allowed to be rotated so set this value to 0
        pNova.SetLayoutOptionFloat(pwsSignatureId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_ROTATION, 0)

        'ZIndex (multiple of 5) - objects with higher ZIndex will be above the ones with smaller ZIndex
        pNova.SetLayoutOptionLong(pwsSignatureId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_ZINDEX, 25)

        'Border
        pNova.SetLayoutOptionBool(pwsSignatureId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_USE_BORDER, 1)
        pNova.SetLayoutOptionLong(pwsSignatureId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_BORDER_POSITION, BorderPosition.BORDER_POSITION_EXTERIOR)
        pNova.SetLayoutOptionLong(pwsSignatureId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_BORDER_WIDTH, 1000)
        pNova.SetLayoutOptionLong(pwsSignatureId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_BORDER_STYLE, BorderStyle.BORDER_STYLE_SOLID)
        pNova.SetLayoutOptionLong(pwsSignatureId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_BORDER_COLOR, System.Drawing.Color.Blue.ToArgb())

        'pages
        pNova.SetLayoutOptionBool(pwsSignatureId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_FIRSTPAGE, 1)
        pNova.SetLayoutOptionBool(pwsSignatureId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_LASTPAGE, 0)
        pNova.SetLayoutOptionBool(pwsSignatureId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_ODDPAGE, 0)
        pNova.SetLayoutOptionBool(pwsSignatureId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_EVENPAGE, 0)
        pNova.SetLayoutOptionBool(pwsSignatureId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_ALLPAGE, 0)
        pNova.SetLayoutOptionBool(pwsSignatureId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_PAGES, 0)
        pNova.SetLayoutOptionString(pwsSignatureId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_PAGERANGE, "")
    End Sub

    '------------------------------------------------------------------------------------------'
    '-------------------------------------  PAGE CONTENT  -------------------------------------'
    '------------------------------------------------------------------------------------------'
    Public Shared Sub AddPageContentOptions(ByVal pNova As NovaPdfOptions90)
        'LAYOUT
        Dim pwsContentId As String = ""
        Dim pwsLayoutId As String = ""
        'Get signature ID
        pNova.GetContentLayout(pwsContentId, pwsLayoutId)

        'metadata
        pNova.SetLayoutOptionString(pwsContentId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_NAME, "Content Layout")
        pNova.SetLayoutOptionString(pwsContentId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_AUTHOR, "Hello World application")
        pNova.SetLayoutOptionString(pwsContentId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_DESCRIPTION, "printed content position on page")

        'anchors
        'percentage or thousands of mm
        pNova.SetLayoutOptionBool(pwsContentId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_LEFTANCHOR_USED, 1)
        pNova.SetLayoutOptionFloat(pwsContentId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_LEFTANCHOR_OFFSET, 15)
        pNova.SetLayoutOptionBool(pwsContentId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_LEFTANCHOR_USE_PERCENT, 1)

        pNova.SetLayoutOptionBool(pwsContentId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_RIGHTANCHOR_USED, 1)
        pNova.SetLayoutOptionFloat(pwsContentId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_RIGHTANCHOR_OFFSET, 15)
        pNova.SetLayoutOptionBool(pwsContentId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_RIGHTANCHOR_USE_PERCENT, 1)

        pNova.SetLayoutOptionBool(pwsContentId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_TOPANCHOR_USED, 1)
        pNova.SetLayoutOptionFloat(pwsContentId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_TOPANCHOR_OFFSET, 15)
        pNova.SetLayoutOptionBool(pwsContentId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_TOPANCHOR_USE_PERCENT, 1)

        pNova.SetLayoutOptionBool(pwsContentId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_BOTTOMANCHOR_USED, 1)
        pNova.SetLayoutOptionFloat(pwsContentId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_BOTTOMANCHOR_OFFSET, 15)
        pNova.SetLayoutOptionBool(pwsContentId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_BOTTOMANCHOR_USE_PERCENT, 1)

        pNova.SetLayoutOptionBool(pwsContentId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_VERTANCHOR_USED, 0)
        pNova.SetLayoutOptionFloat(pwsContentId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_VERTLANCHOR_OFFSET, 0)
        pNova.SetLayoutOptionBool(pwsContentId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_VERTANCHOR_USE_PERCENT, 0)

        pNova.SetLayoutOptionBool(pwsContentId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_HORIZANCHOR_USED, 0)
        pNova.SetLayoutOptionFloat(pwsContentId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_HORIZANCHOR_OFFSET, 0)
        pNova.SetLayoutOptionBool(pwsContentId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_HORIZANCHOR_USE_PERCENT, 0)

        'aspect ratio
        pNova.SetLayoutOptionBool(pwsContentId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_USE_ASPECT_RATIO, 0)
        'allow stretch with anchors when print on a different page size or layout
        pNova.SetLayoutOptionBool(pwsContentId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_ALLOW_STRETCH, 0)

        'Rotation (radians)
        pNova.SetLayoutOptionFloat(pwsContentId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_ROTATION, 0)

        'ZIndex (multiple of 5) - objects with higher ZIndex will be above the ones with smaller ZIndex
        pNova.SetLayoutOptionLong(pwsContentId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_ZINDEX, -1)

        'Border
        pNova.SetLayoutOptionBool(pwsContentId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_USE_BORDER, 1)
        pNova.SetLayoutOptionLong(pwsContentId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_BORDER_POSITION, BorderPosition.BORDER_POSITION_EXTERIOR)
        pNova.SetLayoutOptionLong(pwsContentId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_BORDER_WIDTH, 2000)
        pNova.SetLayoutOptionLong(pwsContentId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_BORDER_STYLE, BorderStyle.BORDER_STYLE_SOLID)
        pNova.SetLayoutOptionLong(pwsContentId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_BORDER_COLOR, System.Drawing.Color.Gray.ToArgb())

        'pages
        pNova.SetLayoutOptionBool(pwsContentId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_FIRSTPAGE, 0)
        pNova.SetLayoutOptionBool(pwsContentId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_LASTPAGE, 0)
        pNova.SetLayoutOptionBool(pwsContentId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_ODDPAGE, 0)
        pNova.SetLayoutOptionBool(pwsContentId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_EVENPAGE, 0)
        pNova.SetLayoutOptionBool(pwsContentId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_ALLPAGE, 1)
        pNova.SetLayoutOptionBool(pwsContentId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_PAGES, 0)
        pNova.SetLayoutOptionString(pwsContentId, pwsLayoutId, NovaOptions.NOVAPDF_LAYOUT_PAGERANGE, "1-2")

    End Sub

    '------------------------------------------------------------------------------------------'
    '--------------------------------  VIEWER OPTIONS  ----------------------------------'
    '------------------------------------------------------------------------------------------'
    Public Shared Sub AddViewerOptions(ByVal pNova As INovaPdfOptions90)
        pNova.SetOptionBool(NovaOptions.NOVAPDF_INFO_VIEWER_ENABLE, 1)

        pNova.SetOptionBool(NovaOptions.NOVAPDF_VIEWOP_HIDE_TOOLBARS, 1)
        pNova.SetOptionBool(NovaOptions.NOVAPDF_VIEWOP_SHOW_DOCTITLE, 1)
        pNova.SetOptionBool(NovaOptions.NOVAPDF_VIEWOP_HIDE_MENUBAR, 1)
        pNova.SetOptionBool(NovaOptions.NOVAPDF_VIEWOP_RESIZE_WINDOW, 1)
        pNova.SetOptionBool(NovaOptions.NOVAPDF_VIEWOP_HIDE_USER, 1)

        pNova.SetOptionLong(NovaOptions.NOVAPDF_VIEWOP_TRANSITION_TYPE, TransitionsTypes.TRANSITION_TYPE_REPLACE)
        pNova.SetOptionLong(NovaOptions.NOVAPDF_VIEWOP_TRANSITION_DIRECTION, TransitionsDirection.TRANSITION_DIR_VERT)
        pNova.SetOptionLong(NovaOptions.NOVAPDF_VIEWOP_TRANSITION_DURATION, 2)
        pNova.SetOptionLong(NovaOptions.NOVAPDF_VIEWOP_PAGE_DURATION, 2)

        pNova.SetOptionBool(NovaOptions.NOVAPDF_VIEWOP_USE_START_PAGE, 1)
        pNova.SetOptionLong(NovaOptions.NOVAPDF_VIEWOP_START_PAGE, 1)

        pNova.SetOptionLong(NovaOptions.NOVAPDF_VIEWOP_ZOOM, PageMagnification.PAGE_MAGN_VIEWERDEFAULT)
        pNova.SetOptionLong(NovaOptions.NOVAPDF_VIEWOP_PAGE_LAYOUT, PageLayout.PAGE_LAY_VIEWERDEF)
        pNova.SetOptionLong(NovaOptions.NOVAPDF_VIEWOP_PAGE_MODE, PageMode.PAGE_MODE_VIEWERDEFAULT)
        pNova.SetOptionLong(NovaOptions.NOVAPDF_VIEWOP_PAGE_SCALING, PageScale.PAGE_SCALE_APPDEFAULT)
        pNova.SetOptionLong(NovaOptions.NOVAPDF_VIEWOP_FULLSCREEN, FullScreenMode.FULLSCREEN_VIEWERDEF)
    End Sub

    '------------------------------------------------------------------------------------------'
    '-------------------------------------  WATERMARK TEXT ------------------------------------'
    '------------------------------------------------------------------------------------------'
    Public Shared Sub AddWatermarkText(ByVal pNova As INovaPdfOptions90)
        Dim pWtmTextId As String = vbNullString
        Dim pLayoutTextId As String = vbNullString

        pNova.AddWatermarkText(pWtmTextId, pLayoutTextId)
        'metadata
        pNova.SetWatermarkTextOptionString(pWtmTextId, NovaOptions.NOVAPDF_WTM_TXT_NAME, "WtmText")
        pNova.SetWatermarkTextOptionString(pWtmTextId, NovaOptions.NOVAPDF_WTM_TXT_AUTHOR, "Hello World application")
        pNova.SetWatermarkTextOptionString(pWtmTextId, NovaOptions.NOVAPDF_WTM_TXT_DESCRIPTION, "text image")
        'watermark text
        pNova.SetWatermarkTextOptionString(pWtmTextId, NovaOptions.NOVAPDF_WTM_TXT_WATERMARK_TEXT, "Watermark")
        'font
        pNova.SetWatermarkTextOptionString(pWtmTextId, NovaOptions.NOVAPDF_WTM_TXT_FONT_NAME, "Arial")
        pNova.SetWatermarkTextOptionLong(pWtmTextId, NovaOptions.NOVAPDF_WTM_TXT_FONT_TYPE, FontType.NOVA_FONTTYPE_TRUETYPE)
        pNova.SetWatermarkTextOptionLong(pWtmTextId, NovaOptions.NOVAPDF_WTM_TXT_FONT_SIZE, 74)
        'font style
        pNova.SetWatermarkTextOptionBool(pWtmTextId, NovaOptions.NOVAPDF_WTM_TXT_FONT_BOLD, 1)
        pNova.SetWatermarkTextOptionBool(pWtmTextId, NovaOptions.NOVAPDF_WTM_TXT_FONT_ITALIC, 1)
        pNova.SetWatermarkTextOptionBool(pWtmTextId, NovaOptions.NOVAPDF_WTM_TXT_FONT_OUTLINE, 0)
        pNova.SetWatermarkTextOptionBool(pWtmTextId, NovaOptions.NOVAPDF_WTM_TXT_FONT_UNDERLINE, 0)
        'font color
        pNova.SetWatermarkTextOptionLong(pWtmTextId, NovaOptions.NOVAPDF_WTM_TXT_FONT_COLOR, System.Drawing.Color.Blue.ToArgb())
        'opacity
        pNova.SetWatermarkTextOptionLong(pWtmTextId, NovaOptions.NOVAPDF_WTM_TXT_OPACITY, 50)
        'visibility
        pNova.SetWatermarkTextOptionBool(pWtmTextId, NovaOptions.NOVAPDF_WTM_TXT_VISIBILITY_VIEW, 1)
        pNova.SetWatermarkTextOptionBool(pWtmTextId, NovaOptions.NOVAPDF_WTM_TXT_VISIBILITY_PRINT, 1)
        pNova.SetWatermarkTextOptionBool(pWtmTextId, NovaOptions.NOVAPDF_WTM_TXT_VISIBILITY_EXPORT, 1)

        'LAYOUT
        'metadata
        pNova.SetLayoutOptionString(pWtmTextId, pLayoutTextId, NovaOptions.NOVAPDF_LAYOUT_NAME, "WtmText Layout")
        pNova.SetLayoutOptionString(pWtmTextId, pLayoutTextId, NovaOptions.NOVAPDF_LAYOUT_AUTHOR, "Hello World application")
        pNova.SetLayoutOptionString(pWtmTextId, pLayoutTextId, NovaOptions.NOVAPDF_LAYOUT_DESCRIPTION, "text watermark position on page")

        'anchors
        'offset is set as percentage or thousands of mm
        pNova.SetLayoutOptionBool(pWtmTextId, pLayoutTextId, NovaOptions.NOVAPDF_LAYOUT_LEFTANCHOR_USED, 1)
        pNova.SetLayoutOptionFloat(pWtmTextId, pLayoutTextId, NovaOptions.NOVAPDF_LAYOUT_LEFTANCHOR_OFFSET, 10)
        pNova.SetLayoutOptionBool(pWtmTextId, pLayoutTextId, NovaOptions.NOVAPDF_LAYOUT_LEFTANCHOR_USE_PERCENT, 1)

        pNova.SetLayoutOptionBool(pWtmTextId, pLayoutTextId, NovaOptions.NOVAPDF_LAYOUT_RIGHTANCHOR_USED, 1)
        pNova.SetLayoutOptionFloat(pWtmTextId, pLayoutTextId, NovaOptions.NOVAPDF_LAYOUT_RIGHTANCHOR_OFFSET, 20000)
        pNova.SetLayoutOptionBool(pWtmTextId, pLayoutTextId, NovaOptions.NOVAPDF_LAYOUT_RIGHTANCHOR_USE_PERCENT, 0)

        pNova.SetLayoutOptionBool(pWtmTextId, pLayoutTextId, NovaOptions.NOVAPDF_LAYOUT_TOPANCHOR_USED, 0)
        pNova.SetLayoutOptionFloat(pWtmTextId, pLayoutTextId, NovaOptions.NOVAPDF_LAYOUT_TOPANCHOR_OFFSET, 0)
        pNova.SetLayoutOptionBool(pWtmTextId, pLayoutTextId, NovaOptions.NOVAPDF_LAYOUT_TOPANCHOR_USE_PERCENT, 0)

        pNova.SetLayoutOptionBool(pWtmTextId, pLayoutTextId, NovaOptions.NOVAPDF_LAYOUT_BOTTOMANCHOR_USED, 0)
        pNova.SetLayoutOptionFloat(pWtmTextId, pLayoutTextId, NovaOptions.NOVAPDF_LAYOUT_BOTTOMANCHOR_OFFSET, 0)
        pNova.SetLayoutOptionBool(pWtmTextId, pLayoutTextId, NovaOptions.NOVAPDF_LAYOUT_BOTTOMANCHOR_USE_PERCENT, 0)

        pNova.SetLayoutOptionBool(pWtmTextId, pLayoutTextId, NovaOptions.NOVAPDF_LAYOUT_VERTANCHOR_USED, 1)
        pNova.SetLayoutOptionFloat(pWtmTextId, pLayoutTextId, NovaOptions.NOVAPDF_LAYOUT_VERTLANCHOR_OFFSET, 0)
        pNova.SetLayoutOptionBool(pWtmTextId, pLayoutTextId, NovaOptions.NOVAPDF_LAYOUT_VERTANCHOR_USE_PERCENT, 0)

        pNova.SetLayoutOptionBool(pWtmTextId, pLayoutTextId, NovaOptions.NOVAPDF_LAYOUT_HORIZANCHOR_USED, 0)
        pNova.SetLayoutOptionFloat(pWtmTextId, pLayoutTextId, NovaOptions.NOVAPDF_LAYOUT_HORIZANCHOR_OFFSET, 0)
        pNova.SetLayoutOptionBool(pWtmTextId, pLayoutTextId, NovaOptions.NOVAPDF_LAYOUT_HORIZANCHOR_USE_PERCENT, 0)

        'aspect ratio
        pNova.SetLayoutOptionBool(pWtmTextId, pLayoutTextId, NovaOptions.NOVAPDF_LAYOUT_USE_ASPECT_RATIO, 1)
        'allow stretch with anchors when print on a different page size or layout
        pNova.SetLayoutOptionBool(pWtmTextId, pLayoutTextId, NovaOptions.NOVAPDF_LAYOUT_ALLOW_STRETCH, 0)

        'ZIndex (multiple of 5) - objects with higher ZIndex will be above the ones with smaller ZIndex
        pNova.SetLayoutOptionLong(pWtmTextId, pLayoutTextId, NovaOptions.NOVAPDF_LAYOUT_ZINDEX, 20)

        'rotation (in radians)
        pNova.SetLayoutOptionFloat(pWtmTextId, pLayoutTextId, NovaOptions.NOVAPDF_LAYOUT_ROTATION, 0.94)

        'Border
        pNova.SetLayoutOptionBool(pWtmTextId, pLayoutTextId, NovaOptions.NOVAPDF_LAYOUT_USE_BORDER, 0)
        pNova.SetLayoutOptionLong(pWtmTextId, pLayoutTextId, NovaOptions.NOVAPDF_LAYOUT_BORDER_POSITION, 0)
        pNova.SetLayoutOptionLong(pWtmTextId, pLayoutTextId, NovaOptions.NOVAPDF_LAYOUT_BORDER_WIDTH, 2000)
        pNova.SetLayoutOptionLong(pWtmTextId, pLayoutTextId, NovaOptions.NOVAPDF_LAYOUT_BORDER_STYLE, BorderStyle.BORDER_STYLE_SOLID)
        pNova.SetLayoutOptionLong(pWtmTextId, pLayoutTextId, NovaOptions.NOVAPDF_LAYOUT_BORDER_COLOR, System.Drawing.Color.Blue.ToArgb())

        'pages
        pNova.SetLayoutOptionBool(pWtmTextId, pLayoutTextId, NovaOptions.NOVAPDF_LAYOUT_FIRSTPAGE, 1)
        pNova.SetLayoutOptionBool(pWtmTextId, pLayoutTextId, NovaOptions.NOVAPDF_LAYOUT_LASTPAGE, 0)
        pNova.SetLayoutOptionBool(pWtmTextId, pLayoutTextId, NovaOptions.NOVAPDF_LAYOUT_ODDPAGE, 0)
        pNova.SetLayoutOptionBool(pWtmTextId, pLayoutTextId, NovaOptions.NOVAPDF_LAYOUT_EVENPAGE, 0)
        pNova.SetLayoutOptionBool(pWtmTextId, pLayoutTextId, NovaOptions.NOVAPDF_LAYOUT_ALLPAGE, 0)
        pNova.SetLayoutOptionBool(pWtmTextId, pLayoutTextId, NovaOptions.NOVAPDF_LAYOUT_PAGES, 0)
        pNova.SetLayoutOptionString(pWtmTextId, pLayoutTextId, NovaOptions.NOVAPDF_LAYOUT_PAGERANGE, "1-2")

    End Sub

    '------------------------------------------------------------------------------------------'
    '-------------------------------------  WATERMARK IMAGE -----------------------------------'
    '------------------------------------------------------------------------------------------'
    Public Shared Sub AddWatermarkImage(ByVal pNova As INovaPdfOptions90)
        Dim pWtmImageId As String = ""
        Dim pLayoutImageId As String = vbNullString

        pNova.AddWatermarkImage(pWtmImageId, pLayoutImageId)
        'metadata
        pNova.SetWatermarkImageOptionString(pWtmImageId, NovaOptions.NOVAPDF_WTM_IMG_NAME, "WtmImage")
        pNova.SetWatermarkImageOptionString(pWtmImageId, NovaOptions.NOVAPDF_WTM_IMG_AUTHOR, "Hello World application")
        pNova.SetWatermarkImageOptionString(pWtmImageId, NovaOptions.NOVAPDF_WTM_IMG_DESCRIPTION, "image watermark")
        'watermark image file
        pNova.SetWatermarkImageOptionString(pWtmImageId, NovaOptions.NOVAPDF_WTM_IMG_IMAGE_FILE_NAME, "copy.png")
        pNova.SetWatermarkImageOptionLong(pWtmImageId, NovaOptions.NOVAPDF_WTM_IMG_IMAGE_LOCATION, LocationType.LOCATION_TYPE_LOCAL)
        pNova.SetWatermarkImageOptionString(pWtmImageId, NovaOptions.NOVAPDF_WTM_IMG_IMAGE_USER_NAME, "")
        pNova.SetWatermarkImageOptionEncryptedString(pWtmImageId, NovaOptions.NOVAPDF_WTM_IMG_IMAGE_USER_PASSWORD, "")
        'use transparency
        pNova.SetWatermarkImageOptionBool(pWtmImageId, NovaOptions.NOVAPDF_WTM_IMG_IMAGE_TRANSPARENCY, 0)
        pNova.SetWatermarkImageOptionLong(pWtmImageId, NovaOptions.NOVAPDF_WTM_IMG_IMAGE_TRANSP_COLOR, System.Drawing.Color.Blue.ToArgb())
        'opacity
        pNova.SetWatermarkImageOptionLong(pWtmImageId, NovaOptions.NOVAPDF_WTM_IMG_IMAGE_OPACITY, 50)
        'color variation
        pNova.SetWatermarkImageOptionLong(pWtmImageId, NovaOptions.NOVAPDF_WTM_IMG_COLOR_VARIATION, 0)
        'visibility
        pNova.SetWatermarkImageOptionBool(pWtmImageId, NovaOptions.NOVAPDF_WTM_IMG_VISIBILITY_VIEW, 1)
        pNova.SetWatermarkImageOptionBool(pWtmImageId, NovaOptions.NOVAPDF_WTM_IMG_VISIBILITY_PRINT, 1)
        pNova.SetWatermarkImageOptionBool(pWtmImageId, NovaOptions.NOVAPDF_WTM_IMG_VISIBILITY_EXPORT, 1)

        'LAYOUT
        'metadata
        pNova.SetLayoutOptionString(pWtmImageId, pLayoutImageId, NovaOptions.NOVAPDF_LAYOUT_NAME, "WtmImage Layout")
        pNova.SetLayoutOptionString(pWtmImageId, pLayoutImageId, NovaOptions.NOVAPDF_LAYOUT_AUTHOR, "Hello World application")
        pNova.SetLayoutOptionString(pWtmImageId, pLayoutImageId, NovaOptions.NOVAPDF_LAYOUT_DESCRIPTION, "image watermark position on page")

        'anchors
        'offset is set as percentage or thousands of mm
        pNova.SetLayoutOptionBool(pWtmImageId, pLayoutImageId, NovaOptions.NOVAPDF_LAYOUT_LEFTANCHOR_USED, 1)
        pNova.SetLayoutOptionFloat(pWtmImageId, pLayoutImageId, NovaOptions.NOVAPDF_LAYOUT_LEFTANCHOR_OFFSET, 10)
        pNova.SetLayoutOptionBool(pWtmImageId, pLayoutImageId, NovaOptions.NOVAPDF_LAYOUT_LEFTANCHOR_USE_PERCENT, 1)

        pNova.SetLayoutOptionBool(pWtmImageId, pLayoutImageId, NovaOptions.NOVAPDF_LAYOUT_RIGHTANCHOR_USED, 1)
        pNova.SetLayoutOptionFloat(pWtmImageId, pLayoutImageId, NovaOptions.NOVAPDF_LAYOUT_RIGHTANCHOR_OFFSET, 50000)
        pNova.SetLayoutOptionBool(pWtmImageId, pLayoutImageId, NovaOptions.NOVAPDF_LAYOUT_RIGHTANCHOR_USE_PERCENT, 0)

        pNova.SetLayoutOptionBool(pWtmImageId, pLayoutImageId, NovaOptions.NOVAPDF_LAYOUT_TOPANCHOR_USED, 0)
        pNova.SetLayoutOptionFloat(pWtmImageId, pLayoutImageId, NovaOptions.NOVAPDF_LAYOUT_TOPANCHOR_OFFSET, 0)
        pNova.SetLayoutOptionBool(pWtmImageId, pLayoutImageId, NovaOptions.NOVAPDF_LAYOUT_TOPANCHOR_USE_PERCENT, 0)

        pNova.SetLayoutOptionBool(pWtmImageId, pLayoutImageId, NovaOptions.NOVAPDF_LAYOUT_BOTTOMANCHOR_USED, 0)
        pNova.SetLayoutOptionFloat(pWtmImageId, pLayoutImageId, NovaOptions.NOVAPDF_LAYOUT_BOTTOMANCHOR_OFFSET, 0)
        pNova.SetLayoutOptionBool(pWtmImageId, pLayoutImageId, NovaOptions.NOVAPDF_LAYOUT_BOTTOMANCHOR_USE_PERCENT, 0)

        pNova.SetLayoutOptionBool(pWtmImageId, pLayoutImageId, NovaOptions.NOVAPDF_LAYOUT_VERTANCHOR_USED, 1)
        pNova.SetLayoutOptionFloat(pWtmImageId, pLayoutImageId, NovaOptions.NOVAPDF_LAYOUT_VERTLANCHOR_OFFSET, 0)
        pNova.SetLayoutOptionBool(pWtmImageId, pLayoutImageId, NovaOptions.NOVAPDF_LAYOUT_VERTANCHOR_USE_PERCENT, 1)

        pNova.SetLayoutOptionBool(pWtmImageId, pLayoutImageId, NovaOptions.NOVAPDF_LAYOUT_HORIZANCHOR_USED, 0)
        pNova.SetLayoutOptionFloat(pWtmImageId, pLayoutImageId, NovaOptions.NOVAPDF_LAYOUT_HORIZANCHOR_OFFSET, 0)
        pNova.SetLayoutOptionBool(pWtmImageId, pLayoutImageId, NovaOptions.NOVAPDF_LAYOUT_HORIZANCHOR_USE_PERCENT, 0)

        'aspect ratio
        pNova.SetLayoutOptionBool(pWtmImageId, pLayoutImageId, NovaOptions.NOVAPDF_LAYOUT_USE_ASPECT_RATIO, 1)
        'allow stretch with anchors when print on a different page size or layout
        pNova.SetLayoutOptionBool(pWtmImageId, pLayoutImageId, NovaOptions.NOVAPDF_LAYOUT_ALLOW_STRETCH, 0)

        'Width / Height
        pNova.SetLayoutOptionFloat(pWtmImageId, pLayoutImageId, NovaOptions.NOVAPDF_LAYOUT_OBJECT_WIDTH, 100000)
        pNova.SetLayoutOptionFloat(pWtmImageId, pLayoutImageId, NovaOptions.NOVAPDF_LAYOUT_OBJECT_HEIGHT, 100000)

        'rotation (in radians)
        pNova.SetLayoutOptionFloat(pWtmImageId, pLayoutImageId, NovaOptions.NOVAPDF_LAYOUT_ROTATION, 0)

        'ZIndex (multiple of 5) - objects with higher ZIndex will be above the ones with smaller ZIndex
        pNova.SetLayoutOptionLong(pWtmImageId, pLayoutImageId, NovaOptions.NOVAPDF_LAYOUT_ZINDEX, 15)

        'Border
        pNova.SetLayoutOptionBool(pWtmImageId, pLayoutImageId, NovaOptions.NOVAPDF_LAYOUT_USE_BORDER, 1)
        pNova.SetLayoutOptionLong(pWtmImageId, pLayoutImageId, NovaOptions.NOVAPDF_LAYOUT_BORDER_POSITION, 0)
        pNova.SetLayoutOptionLong(pWtmImageId, pLayoutImageId, NovaOptions.NOVAPDF_LAYOUT_BORDER_WIDTH, 2000)
        pNova.SetLayoutOptionLong(pWtmImageId, pLayoutImageId, NovaOptions.NOVAPDF_LAYOUT_BORDER_STYLE, BorderStyle.BORDER_STYLE_SOLID)
        pNova.SetLayoutOptionLong(pWtmImageId, pLayoutImageId, NovaOptions.NOVAPDF_LAYOUT_BORDER_COLOR, System.Drawing.Color.Green.ToArgb())

        'pages
        pNova.SetLayoutOptionBool(pWtmImageId, pLayoutImageId, NovaOptions.NOVAPDF_LAYOUT_FIRSTPAGE, 0)
        pNova.SetLayoutOptionBool(pWtmImageId, pLayoutImageId, NovaOptions.NOVAPDF_LAYOUT_LASTPAGE, 0)
        pNova.SetLayoutOptionBool(pWtmImageId, pLayoutImageId, NovaOptions.NOVAPDF_LAYOUT_ODDPAGE, 0)
        pNova.SetLayoutOptionBool(pWtmImageId, pLayoutImageId, NovaOptions.NOVAPDF_LAYOUT_EVENPAGE, 0)
        pNova.SetLayoutOptionBool(pWtmImageId, pLayoutImageId, NovaOptions.NOVAPDF_LAYOUT_ALLPAGE, 1)
        pNova.SetLayoutOptionBool(pWtmImageId, pLayoutImageId, NovaOptions.NOVAPDF_LAYOUT_PAGES, 0)
        pNova.SetLayoutOptionString(pWtmImageId, pLayoutImageId, NovaOptions.NOVAPDF_LAYOUT_PAGERANGE, "1-2")

    End Sub

    '------------------------------------------------------------------------------------------'
    '-------------------------------------  ADVANCED OPTIONS -----------------------------------------'
    '------------------------------------------------------------------------------------------'
    Public Shared Sub AddAdvancedOptions(ByVal pNova As INovaPdfOptions90)
        'advanced options
        pNova.SetOptionBool(NovaOptions.NOVAPDF_ADV_OPTIMIZE_TEXT, 1)
        pNova.SetOptionBool(NovaOptions.NOVAPDF_ADV_OPTIMIZE_IMAGE, 1)
        pNova.SetOptionBool(NovaOptions.NOVAPDF_ADV_SILENT_PRINT, 1)
        pNova.SetOptionBool(NovaOptions.NOVAPDF_ADV_CORRECT_FILLCOLOR, 0)
        pNova.SetOptionBool(NovaOptions.NOVAPDF_ADV_CORRECT_LINEWIDTH, 1)
        pNova.SetOptionBool(NovaOptions.NOVAPDF_ADV_IGNORE_EMPTY_PAGES, 1)
        pNova.SetOptionBool(NovaOptions.NOVAPDF_ADV_VERIFY_FILE, 0)
        pNova.SetOptionBool(NovaOptions.NOVAPDF_ADV_COPIES_ADD_PAGE, 0)
        pNova.SetOptionBool(NovaOptions.NOVAPDF_ADV_COPIES_ONLY_ODD, 0)
    End Sub

    '------------------------------------------------------------------------------------------'
    '-------------------------------------  PROFILE OPTIONS -----------------------------------------'
    '------------------------------------------------------------------------------------------'
    Public Shared Sub AddProfileOptions(ByVal pNova As INovaPdfOptions90)
        'profile author
        pNova.SetOptionString(NovaOptions.NOVAPDF_PROFILE_AUTHOR, "Hello World application")
        'profile description
        pNova.SetOptionString(NovaOptions.NOVAPDF_PROFILE_DESCRIPTION, "Hello World application settings")
        'PDF version
        pNova.SetOptionLong(NovaOptions.NOVAPDF_PDF_VERSION, NovaVersions.PDF_VERSION_17)
        'generate PDFA compliant docs
        pNova.SetOptionBool(NovaOptions.NOVAPDF_PDFA, 1)
        'choose between PDFA/1a or PDFA/1b standards
        pNova.SetOptionLong(NovaOptions.NOVAPDF_PDFA_VERSION, PDFAType.PDFA_1A)
        'generate linearized docs
        pNova.SetOptionBool(NovaOptions.NOVAPDF_PDF_LINEARIZE, 0)
    End Sub

    '------------------------------------------------------------------------------------------'
    '---------------------------------------  SAVE OPTIONS  -----------------------------------'
    '------------------------------------------------------------------------------------------'
    Public Shared Sub AddSaveOptions(ByVal pNova As INovaPdfOptions90)
        'choose where to save the pdf file - on the client computer or on the server computer
        pNova.SetOptionLong(NovaOptions.NOVAPDF_SAVE_LOCATION, LocationType.LOCATION_TYPE_LOCAL)
        'what type of save dialog to be shown (none, simple or extended)
        pNova.SetOptionLong(NovaOptions.NOVAPDF_SAVE_PROMPT_TYPE, SaveDlgType.PROMPT_SAVE_NONE)
        'you may choose a predefined folder like "My Documents"
        'pNova.SetOptionLong(NovaOptions.NOVAPDF_SAVE_FOLDER_TYPE, SaveFolder.SAVEFOLDER_MYDOCUMENTS)
        'or choose a custom folder to save the files
        pNova.SetOptionLong(NovaOptions.NOVAPDF_SAVE_FOLDER_TYPE, SaveFolder.SAVEFOLDER_CUSTOM)
        pNova.SetOptionString(NovaOptions.NOVAPDF_SAVE_FOLDER, "C:\temp")
        'if the folder is on a network path, set network user name and password to access this path
        'pNova.SetOptionString(NovaOptions.NOVAPDF_SAVE_USER_NAME, "")
        'pNova.SetOptionEncryptedString(NovaOptions.NOVAPDF_SAVE_USER_PASSWORD, "")
        'set a macro for the file name
        pNova.SetOptionString(NovaOptions.NOVAPDF_SAVE_FILE_NAME, "test_[N]")
        'in case a file with the same name exists in the selected folder, choose what to do
        pNova.SetOptionLong(NovaOptions.NOVAPDF_SAVE_FILEEXIST_ACTION, SaveFileConflictType.FILE_CONFLICT_STRATEGY_OVERWRITE)
    End Sub
    '------------------------------------------------------------------------------------------//
    '---------------------------------------  MERGE OPTIONS  -----------------------------------//
    '------------------------------------------------------------------------------------------//
    Public Shared Sub AddMergeOptions(ByVal pNova As INovaPdfOptions90)
        'Enable Merge PDF option
        pNova.SetOptionBool(NovaOptions.NOVAPDF_MERGE_ENABLE, 1)
        'show a dialog before printing each document to enter merge options
        pNova.SetOptionBool(NovaOptions.NOVAPDF_SHOW_MERGE_DLG, 0)

        'merge file type: 
        'NOVAPDF_MERGE_IF_FILE_EXISTS = TRUE - if a PDF with the same name exists in the Save folder, the new pages will be appended
        'this option is useful when printing multiple documents and wish to add them to the same PDF file
        'NOVAPDF_MERGE_IF_FILE_EXISTS = FALSE - merge with another file specified in NOVAPDF_MERGE_OTHER_FILE option
        'this is useful if for instance you wish to add a title page at the beginning of each PDF you create
        pNova.SetOptionBool(NovaOptions.NOVAPDF_MERGE_IF_FILE_EXISTS, 1)

        'if merge with other PDF file (NOVAPDF_MERGE_IF_FILE_EXISTS = FALSE) 
        'choose where is existing PDF file located - on the client computer or on the server computer
        pNova.SetOptionLong(NovaOptions.NOVAPDF_MERGE_LOCATION, LocationType.LOCATION_TYPE_LOCAL)

        'Existing PDF for merge
        pNova.SetOptionString(NovaOptions.NOVAPDF_MERGE_OTHER_FILE, "C:\temp\title.pdf")

        'if the folder is on a network path, set network user name and password to access this path
        pNova.SetOptionString(NovaOptions.NOVAPDF_MERGE_USER, "")
        pNova.SetOptionEncryptedString(NovaOptions.NOVAPDF_MERGE_PASSWORD, "")

        'Merge type: append at the end, insert at the beginning or insert at a specified page number
        pNova.SetOptionLong(NovaOptions.NOVAPDF_MERGE_TYPE, MergeType.MERGE_TYPE_APPEND)

        'if merge type is MERGE_TYPE_INSERT_PAGENO set where in the pdf to insert the pages
        pNova.SetOptionLong(NovaOptions.NOVAPDF_MERGE_INSERT_PAGENO, 3)

        'in case existing pdf file is password protected, enter the password here
        pNova.SetOptionEncryptedString(NovaOptions.NOVAPDF_MERGE_APPEND_PASSWORD, "my password")
    End Sub

    '------------------------------------------------------------------------------------------'
    '-----------------------------------------  ACTIONS  --------------------------------------'
    '------------------------------------------------------------------------------------------'
    Public Shared Sub AddAction_OpenFile(ByVal pNova As INovaPdfOptions90, ByRef strActionID As String)
        Dim nActionIndex As Integer
        pNova.AddAction(ActionType.NOVA_ACTION_OPEN, "My Open Action", strActionID, nActionIndex)
        'Execute this action	
        pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_ACTION_EXECUTE, 1)
        'if action fails, execute the other actions
        pNova.SetActionOptionLong(strActionID, NovaOptions.NOVAPDF_ACTION_FAIL, ActionFailType.NOVA_ACTION_FAIL_CONTINUE)
        'open original pdf file (the pdf file that was generated now)
        pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_ACTION_OPEN_OPENORIGINAL, 1)
        ' if NOVAPDF_ACTION_OPEN_OPENORIGINAL is FALSE, open custom file; 
        'pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_ACTION_OPEN_OPENORIGINAL, 0)
        'pNova.SetActionOptionString(strActionID, NovaOptions.NOVAPDF_ACTION_OPEN_CUSTOMSOURCE, "C:\temp\<file to open>.pdf")
    End Sub
    '------------------------------------------------------------------------------------------'
    Public Shared Sub AddAction_RunApplication(ByVal pNova As INovaPdfOptions90, ByRef strActionID As String)
        Dim nActionIndex As Integer
        pNova.AddAction(ActionType.NOVA_ACTION_RUN, "My Run Action", strActionID, nActionIndex)
        'Execute this action	
        pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_ACTION_EXECUTE, 1)
        'if action fails, do not execute the other actions
        pNova.SetActionOptionLong(strActionID, NovaOptions.NOVAPDF_ACTION_FAIL, ActionFailType.NOVA_ACTION_FAIL_STOP)
        'set application to be run
        pNova.SetActionOptionString(strActionID, NovaOptions.NOVAPDF_ACTION_RUN_APPLICATION, "C:\Program Files (x86)\Adobe\Acrobat Reader DC\Reader\AcroRd32.exe")
        'command line parameters for the application; %1 will be replaced with the generated PDF file name (with full path)
        pNova.SetActionOptionString(strActionID, NovaOptions.NOVAPDF_ACTION_RUN_CMDLINE_PARAMETERS, "%1")
        'user and password to connect to the application folder
        'pNova.SetActionOptionString(strActionID, NovaOptions.NOVAPDF_ACTION_RUN_USER_NAME, "user")
        'pNova.SetActionOptionEncryptedString(strActionID, NovaOptions.NOVAPDF_ACTION_RUN_USER_PASSWORD, "password")
        'Show or hide application window
        pNova.SetActionOptionLong(strActionID, NovaOptions.NOVAPDF_ACTION_RUN_SHOW, RunAs.RUN_NORMAL)
        'Start the application as child process or with ShellExecute
        pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_ACTION_RUN_SHELL, 1)
    End Sub
    '------------------------------------------------------------------------------------------'
    Public Shared Sub AddAction_CopyFile(ByVal pNova As INovaPdfOptions90, ByRef strActionID As String)
        Dim nActionIndex As Integer
        'add a copy file action before generating pdf (if for instance we wish to append to an existing pdf)
        pNova.AddAction(ActionType.NOVA_ACTION_COPY, "My Copy action", strActionID, nActionIndex)
        'Execute this action	
        pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_ACTION_EXECUTE, 1)
        'if action fails, do not execute the other actions
        pNova.SetActionOptionLong(strActionID, NovaOptions.NOVAPDF_ACTION_FAIL, ActionFailType.NOVA_ACTION_FAIL_STOP)
        'copy source: custom file
        pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_ACTION_COPY_SOURCECOPYORIGINAL, 0)
        pNova.SetActionOptionString(strActionID, NovaOptions.NOVAPDF_ACTION_COPY_CUSTOMSOURCE, "%PROGRAMDATA%\Softland\novaPDF 9\nPdfSdk9_Softland\DefaultFiles\invoice-text.pdf")
        'user and password to connect to pdf file
        'pNova.SetActionOptionString(strActionID, NovaOptions.NOVAPDF_ACTION_COPY_SOURCEUSERNAME, "user")
        'pNova.SetActionOptionEncryptedString(strActionID, NovaOptions.NOVAPDF_ACTION_COPY_SOURCEPASSWORD, "password")
        'copy destination: new created pdf that will be appended
        pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_ACTION_COPY_DESTCOPYORIGINAL, 1)
        ' if NOVAPDF_ACTION_COPY_DESTCOPYORIGINAL is FALSE, set folder and file name for destination
        'pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_ACTION_COPY_DESTCOPYORIGINAL, 0)
        'pNova.SetActionOptionString(strActionID, NovaOptions.NOVAPDF_ACTION_COPY_CUSTOMDESTFOLDER, "C:\temp")
        'pNova.SetActionOptionString(strActionID, NovaOptions.NOVAPDF_ACTION_COPY_CUSTOMDESTFILE, "<my pdf file>.pdf")
        'pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_ACTION_COPY_DESTKEEPFILENAME, 0);
        'user and password to connect to folder
        'pNova.SetActionOptionString(strActionID, NovaOptions.NOVAPDF_ACTION_COPY_DESTUSERNAME, "user")
        'pNova.SetActionOptionEncryptedString(strActionID, NovaOptions.NOVAPDF_ACTION_COPY_DESTPASSWORD, "password")
    End Sub
    '------------------------------------------------------------------------------------------'
    Public Shared Sub AddAction_DeleteFile(ByVal pNova As INovaPdfOptions90, ByRef strActionID As String)
        Dim nActionIndex As Integer
        pNova.AddAction(ActionType.NOVA_ACTION_DELETE, "My Delete Action", strActionID, nActionIndex)
        'Execute this action	
        pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_ACTION_EXECUTE, 1)
        'if action fails, execute the other actions
        pNova.SetActionOptionLong(strActionID, NovaOptions.NOVAPDF_ACTION_FAIL, ActionFailType.NOVA_ACTION_FAIL_CONTINUE)
        'delete original pdf file
        pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_ACTION_DELETE_DELETEORIGINAL, 1)
        ' if NOVAPDF_ACTION_DELETE_DELETEORIGINAL is TRUS, set the name of file to delete
        'pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_ACTION_DELETE_DELETEORIGINAL, 0)
        'pNova.SetActionOptionString(strActionID, NovaOptions.NOVAPDF_ACTION_DELETE_CUSTOMSOURCE, "C:\temp\<file to delete>.pdf")
        'user and password to connect to pdf file
        'pNova.SetActionOptionString(strActionID, NovaOptions.NOVAPDF_ACTION_DELETE_USERNAME, "user")
        'pNova.SetActionOptionEncryptedString(strActionID, NovaOptions.NOVAPDF_ACTION_DELETE_PASSWORD, "password")
    End Sub
    '------------------------------------------------------------------------------------------'
    Public Shared Sub AddAction_EmailMAPI(ByVal pNova As INovaPdfOptions90, ByRef strActionID As String)
        Dim nActionIndex As Integer
        pNova.AddAction(ActionType.NOVA_ACTION_EMAILMAPI, "My Email MAPI Action", strActionID, nActionIndex)
        'Execute this action	
        pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_ACTION_EXECUTE, 1)
        'if action fails, execute the other actions
        pNova.SetActionOptionLong(strActionID, NovaOptions.NOVAPDF_ACTION_FAIL, ActionFailType.NOVA_ACTION_FAIL_CONTINUE)
        'Show dialog before printing each document to enter email address
        pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_ACTION_SHOW_DIALOG, 0)
        'Email MAPI method (show dialog or send automatically)
        pNova.SetActionOptionLong(strActionID, NovaOptions.NOVAPDF_EMAILMAPI_SENDEMAIL, SendEmailMethod.EMAIL_TYPE_MAPI_DLG)
        'Email details
        pNova.SetActionOptionString(strActionID, NovaOptions.NOVAPDF_EMAILMAPI_SUBJECT, "my email subject")
        pNova.SetActionOptionString(strActionID, NovaOptions.NOVAPDF_EMAILMAPI_BODY, "my email body")
        'flag if the addresses should be looked up in the address book
        pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_EMAILMAPI_LOOKUP_ADDRESS, 0)
        'flag - if the PDF should be sent as zip compression
        pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_EMAILMAPI_COMPRESS, 0)
        'flag - change the extension for the compressed zip attachment
        pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_EMAILMAPI_CHANGEEXTENSION, 0)
        pNova.SetActionOptionString(strActionID, NovaOptions.NOVAPDF_EMAILMAPI_EXTENSION, "txt")
        'attach the PDF to the email
        pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_EMAILMAPI_ATTACH_PDF, 1)
        'attach other file to the email
        pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_EMAILMAPI_ATTACH_OTHER_FILES, 0)
        pNova.SetActionOptionString(strActionID, NovaOptions.NOVAPDF_EMAILMAPI_OTHER_FILES, "path to other files to attach")
        'Set receipient addresses 
        pNova.SetActionOptionString(strActionID, NovaOptions.NOVAPDF_EMAILMAPI_FROM, "from_adress")
        pNova.SetActionOptionString(strActionID, NovaOptions.NOVAPDF_EMAILMAPI_TO, "to_adress")
        pNova.SetActionOptionString(strActionID, NovaOptions.NOVAPDF_EMAILMAPI_CC, "cc_adress")
        pNova.SetActionOptionString(strActionID, NovaOptions.NOVAPDF_EMAILMAPI_BCC, "bcc_adress")
    End Sub
    '------------------------------------------------------------------------------------------'
    Public Shared Sub AddAction_EmailSMTP(ByVal pNova As INovaPdfOptions90, ByRef strActionID As String)
        Dim nActionIndex As Integer

        pNova.AddAction(ActionType.NOVA_ACTION_EMAILSMTP, "My Email SMTP Action", strActionID, nActionIndex)
        'Execute this action	
        pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_ACTION_EXECUTE, 1)
        'if action fails, execute the other actions
        pNova.SetActionOptionLong(strActionID, NovaOptions.NOVAPDF_ACTION_FAIL, ActionFailType.NOVA_ACTION_FAIL_CONTINUE)
        'Show dialog before printing each document to enter email address
        pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_ACTION_SHOW_DIALOG, 0)
        'SMTP server name and port
        pNova.SetActionOptionString(strActionID, NovaOptions.NOVAPDF_EMAILSMTP_SERVER, "my SMTP server name")
        pNova.SetActionOptionString(strActionID, NovaOptions.NOVAPDF_EMAILSMTP_PORT, "25")
        'SMTP account
        pNova.SetActionOptionString(strActionID, NovaOptions.NOVAPDF_EMAILSMTP_ACCOUNT, "my SMTP user name")
        pNova.SetActionOptionEncryptedString(strActionID, NovaOptions.NOVAPDF_EMAILSMTP_PASSWORD, "my SMTP password")
        'flags, if the SMTP server requires Authentification or SSL (1 or 0)
        pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_EMAILSMTP_SSL, 0)
        pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_EMAILSMTP_TLS, 0)
        pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_EMAILSMTP_AUTHENTICATION, 0)
        'Email details
        pNova.SetActionOptionString(strActionID, NovaOptions.NOVAPDF_EMAILSMTP_SUBJECT, "my email subject")
        pNova.SetActionOptionString(strActionID, NovaOptions.NOVAPDF_EMAILSMTP_BODY, "my email body")
        'flag - if the PDF should be sent as zip compression
        pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_EMAILSMTP_COMPRESS, 0)
        'flag - change the extension for the compressed zip attachment
        pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_EMAILSMTP_CHANGEEXTENSION, 0)
        pNova.SetActionOptionString(strActionID, NovaOptions.NOVAPDF_EMAILSMTP_EXTENSION, "txt")
        'attach the PDF to the email
        pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_EMAILSMTP_ATTACH_PDF, 1)
        'attach other file to the email
        pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_EMAILSMTP_ATTACH_OTHER_FILES, 0)
        pNova.SetActionOptionString(strActionID, NovaOptions.NOVAPDF_EMAILSMTP_OTHER_FILES, "path to other files to attach")
        'Set receipient addresses 
        pNova.SetActionOptionString(strActionID, NovaOptions.NOVAPDF_EMAILSMTP_FROM, "from adress")
        pNova.SetActionOptionString(strActionID, NovaOptions.NOVAPDF_EMAILSMTP_TO, "to adress")
        pNova.SetActionOptionString(strActionID, NovaOptions.NOVAPDF_EMAILSMTP_CC, "cc adress")
        pNova.SetActionOptionString(strActionID, NovaOptions.NOVAPDF_EMAILSMTP_BCC, "bcc adress")
    End Sub
    '------------------------------------------------------------------------------------------'
    Public Shared Sub AddAction_FTP(ByVal pNova As INovaPdfOptions90, ByRef strActionID As String)
        Dim nActionIndex As Integer

        pNova.AddAction(ActionType.NOVA_ACTION_FTP, "My FTP Action", strActionID, nActionIndex)
        'Execute this action	
        pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_ACTION_EXECUTE, 1)
        'if action fails, execute the other actions
        pNova.SetActionOptionLong(strActionID, NovaOptions.NOVAPDF_ACTION_FAIL, ActionFailType.NOVA_ACTION_FAIL_CONTINUE)
        'original pdf file
        pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_ACTION_FTP_SOURCEORIGINAL, 1)
        ' if NOVAPDF_ACTION_FTP_SOURCEORIGINAL is FALSE, set the name of file to upload
        'pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_ACTION_FTP_SOURCEORIGINAL, 0)
        'pNova.SetActionOptionString(strActionID, NovaOptions.NOVAPDF_ACTION_FTP_SOURCEFILE, "C:\temp\<file to upload>")
        'user and password to connect to source pdf file
        pNova.SetActionOptionString(strActionID, NovaOptions.NOVAPDF_ACTION_FTP_SOURCEUSER, "user")
        pNova.SetActionOptionEncryptedString(strActionID, NovaOptions.NOVAPDF_ACTION_FTP_SOURCEPASSWORD, "password")
        'destination folder
        pNova.SetActionOptionString(strActionID, NovaOptions.NOVAPDF_ACTION_FTP_DESTFOLDER, "test")
        'use the same file name
        pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_ACTION_FTP_KEEPFILENAME, 1)
        'or change file name
        'pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_ACTION_FTP_KEEPFILENAME, 0)
        'pNova.SetActionOptionString(strActionID, NovaOptions.NOVAPDF_ACTION_FTP_DESTFILE, "<new file name>")
        'ftp server
        pNova.SetActionOptionString(strActionID, NovaOptions.NOVAPDF_ACTION_FTP_SERVER, "192.168.1.1")
        pNova.SetActionOptionString(strActionID, NovaOptions.NOVAPDF_ACTION_FTP_PORT, "21")
        pNova.SetActionOptionString(strActionID, NovaOptions.NOVAPDF_ACTION_FTP_USERNAME, "user")
        pNova.SetActionOptionEncryptedString(strActionID, NovaOptions.NOVAPDF_ACTION_FTP_PASSWORD, "password")
        'host
        pNova.SetActionOptionString(strActionID, NovaOptions.NOVAPDF_ACTION_FTP_HOST, "")
        pNova.SetActionOptionString(strActionID, NovaOptions.NOVAPDF_ACTION_FTP_HOSTPORT, "80")
        pNova.SetActionOptionString(strActionID, NovaOptions.NOVAPDF_ACTION_FTP_HOSTUSERNAME, "")
        pNova.SetActionOptionEncryptedString(strActionID, NovaOptions.NOVAPDF_ACTION_FTP_HOSTPASSWORD, "")
        'ftp options
        pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_ACTION_FTP_UPLOADSPEEDCHECK, 0)
        pNova.SetActionOptionLong(strActionID, NovaOptions.NOVAPDF_ACTION_FTP_UPLOADSPEED, 200)
        pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_ACTION_FTP_USEPASSIVEMODE, 0)
        pNova.SetActionOptionLong(strActionID, NovaOptions.NOVAPDF_ACTION_FTP_TIMEOUT, 30)
        pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_ACTION_FTP_RETRY, 0)
        pNova.SetActionOptionLong(strActionID, NovaOptions.NOVAPDF_ACTION_FTP_RETRYVALUE, 2)
        pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_ACTION_FTP_WAIT, 0)
        pNova.SetActionOptionLong(strActionID, NovaOptions.NOVAPDF_ACTION_FTP_WAITVALUE, 60)
        pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_ACTION_FTP_SSL, 0)
        pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_ACTION_FTP_USEPROXY, 0)
    End Sub
    '------------------------------------------------------------------------------------------'
    Public Shared Sub AddAction_SFTP(ByVal pNova As INovaPdfOptions90, ByRef strActionID As String)
        Dim nActionIndex As Integer

        pNova.AddAction(ActionType.NOVA_ACTION_SFTP, "My SFTP Action", strActionID, nActionIndex)
        'Execute this action	
        pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_ACTION_EXECUTE, 1)
        'if action fails, execute the other actions
        pNova.SetActionOptionLong(strActionID, NovaOptions.NOVAPDF_ACTION_FAIL, ActionFailType.NOVA_ACTION_FAIL_CONTINUE)
        'upload original pdf file
        pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_ACTION_SFTP_SOURCEORIGINAL, 1)
        'or upload a custom file
        'pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_ACTION_SFTP_SOURCEORIGINAL, 0)
        'pNova.SetActionOptionString(strActionID, NovaOptions.NOVAPDF_ACTION_SFTP_SOURCEFILE, "C:\temp\<custom file>")
        'user and password to connect to source pdf file
        pNova.SetActionOptionString(strActionID, NovaOptions.NOVAPDF_ACTION_SFTP_SOURCEUSER, "")
        pNova.SetActionOptionEncryptedString(strActionID, NovaOptions.NOVAPDF_ACTION_SFTP_SOURCEPASSWORD, "")
        'destination folder
        pNova.SetActionOptionString(strActionID, NovaOptions.NOVAPDF_ACTION_SFTP_DESTFOLDER, "test")
        ' destination file name - use the same file name
        pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_ACTION_SFTP_KEEPFILENAME, 1)
        ' or change file name
        'pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_ACTION_SFTP_KEEPFILENAME, 0)
        'pNova.SetActionOptionString(strActionID, NovaOptions.NOVAPDF_ACTION_SFTP_DESTFILE, "<new file name>")
        'sftp server
        pNova.SetActionOptionString(strActionID, NovaOptions.NOVAPDF_ACTION_SFTP_SERVER, "192.168.1.1")
        pNova.SetActionOptionString(strActionID, NovaOptions.NOVAPDF_ACTION_SFTP_PORT, "22")
        pNova.SetActionOptionString(strActionID, NovaOptions.NOVAPDF_ACTION_SFTP_USERNAME, "user")
        pNova.SetActionOptionEncryptedString(strActionID, NovaOptions.NOVAPDF_ACTION_SFTP_PASSWORD, "password")
        'sftp options
        pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_ACTION_SFTP_USEPRIVATEKEY, 0)
        pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_ACTION_SFTP_UPLOADSPEEDCHECK, 0)
        pNova.SetActionOptionLong(strActionID, NovaOptions.NOVAPDF_ACTION_SFTP_UPLOADSPEED, 200)
        pNova.SetActionOptionLong(strActionID, NovaOptions.NOVAPDF_ACTION_SFTP_TIMEOUT, 30)
        pNova.SetActionOptionString(strActionID, NovaOptions.NOVAPDF_ACTION_SFTP_KEYFILENAME, "")
        pNova.SetActionOptionEncryptedString(strActionID, NovaOptions.NOVAPDF_ACTION_SFTP_KEYFILEPASSWORD, "")
        pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_ACTION_SFTP_RETRY, 0)
        pNova.SetActionOptionLong(strActionID, NovaOptions.NOVAPDF_ACTION_SFTP_RETRYVALUE, 2)
        pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_ACTION_SFTP_WAIT, 0)
        pNova.SetActionOptionLong(strActionID, NovaOptions.NOVAPDF_ACTION_SFTP_WAITVALUE, 30)
        'Authentications
        pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_ACTION_SFTP_AUTH_PUBLICKEY, 1)
        pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_ACTION_SFTP_AUTH_PASSWORD, 1)
        pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_ACTION_SFTP_AUTH_KEYBOARDINT, 1)
        'Ciphers
        pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_ACTION_SFTP_CIPHER_AES256CTR, 1)
        pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_ACTION_SFTP_CIPHER_3DESCBC, 1)
        pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_ACTION_SFTP_CIPHER_AES128CBC, 1)
        pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_ACTION_SFTP_CIPHER_AES192CBC, 1)
        pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_ACTION_SFTP_CIPHER_AES256CBC, 1)
        pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_ACTION_SFTP_CIPHER_BLOWFISHCBC, 1)
        pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_ACTION_SFTP_CIPHER_TWOFISHCBC, 1)
        pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_ACTION_SFTP_CIPHER_TWOFISH192CBC, 1)
        pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_ACTION_SFTP_CIPHER_TWOFISH128CBC, 1)
        pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_ACTION_SFTP_CIPHER_TWOFISH256CBC, 1)
        pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_ACTION_SFTP_CIPHER_ARCFOUR, 1)
        pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_ACTION_SFTP_CIPHER_ARCFOUR128, 1)
        pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_ACTION_SFTP_CIPHER_ARCFOUR256, 1)
        pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_ACTION_SFTP_CIPHER_CAST128CBC, 1)
        pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_ACTION_SFTP_CIPHER_AES128CTR, 1)
        pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_ACTION_SFTP_CIPHER_AES192CTR, 1)
        'MACs	
        pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_ACTION_SFTP_MAC_MD5, 1)
        pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_ACTION_SFTP_MAC_SHA1, 1)
        pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_ACTION_SFTP_MAC_SHA2256, 1)
        pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_ACTION_SFTP_MAC_SHA225696, 1)
        pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_ACTION_SFTP_MAC_RIPEMD160, 1)
        pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_ACTION_SFTP_MAC_RIPEMD160OPENSSH, 1)
        pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_ACTION_SFTP_MAC_MD596, 1)
        pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_ACTION_SFTP_MAC_SHA196, 1)
        'Compression
        pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_ACTION_SFTP_COMPRESS_NONE, 1)
        pNova.SetActionOptionBool(strActionID, NovaOptions.NOVAPDF_ACTION_SFTP_COMPRESS_ZLIBOPENSSH, 1)
    End Sub
    '------------------------------------------------------------------------------------------'
    '------------------------------------------------------------------------------------------'
    Public Shared Sub DeleteActionType(ByVal pNova As INovaPdfOptions90, ByVal nActionType As Integer)
        Dim strActionID As String = ""
        Dim nCount, n, nType As Integer
        pNova.GetActionCount(nCount)
        n = nCount - 1
        'parse actions
        While (n >= 0)
            pNova.GetAction(n, strActionID, nType)
            'delete all actions of given type
            If (nType = nActionType) Then
                pNova.DeleteAction(strActionID)
            End If
            n = n - 1
        End While
    End Sub
    '------------------------------------------------------------------------------------------'
    ' There can be added actions to manipulate files (copy or delete), to run applications (open or run), 
    '   to send the pdf by email (MAPI or SMTP) or to upload the pdf (with FTP or SFTP)
    ' Next there is an exemple of actions that append the new generated pdf to an existing one, 
    '  send it by email and then deletes the pdf. There are used next actions
    ' - a copy action that copies the original pdf that must be appended; this action will be performed before generating the pdf
    ' - an email MAPI action
    ' - a delete action
    '------------------------------------------------------------------------------------------'
    Public Shared Sub AddActionsOptions(ByVal pNova As INovaPdfOptions90)
        Dim strActionID As String = ""
        'DO NOT OPEN PDF 
        'an Open action is added by default in the profile - delete or disable this open action
        pNova.DisableActionType(ActionType.NOVA_ACTION_OPEN)

        'add action copy; will copy the file that will be appended 
        AddAction_CopyFile(pNova, strActionID)
        'set action index to 1 - first in the actions list - so it is executed before generate pdf
        pNova.SetActionOptionLong(strActionID, NovaOptions.NOVAPDF_ACTION_INDEX, 0)

        'modify save options to append to existing pdf
        AddSaveOptions(pNova)

        'add action email MAPI - open default rmail client send email window
        AddAction_EmailMAPI(pNova, strActionID)

        'add action delete - delete created pdf file
        AddAction_DeleteFile(pNova, strActionID)

        'add action run application
        'AddAction_RunApplication(pNova, strActionID)

        'add action open file
        'AddAction_OpenFile(pNova, strActionID)

        'add action email SMTP
        'AddAction_EmailSMTP(pNova, strActionID)

        'add action FTP
        'AddAction_FTP(pNova, strActionID)

        'add action SFTP
        'AddAction_SFTP(pNova, strActionID)


    End Sub
End Class
